unit CDDBConnect;

interface

uses
  IdHTTP, Classes;

{ 07/11/2001 V1.02: Ajout dela mthode RefreshCD qui actualise les proprits
                    CDDBDiscIdHex, CDTotalTrackCount, CDTotalLenght
                    ajout des mthodes publiques:
                    GetTrackNameList(Liste: TStrings) => Liste des pistes
                    et GetTrackExtList((Liste: TStrings) => Extension des pistes
                    et des propits AlbumName, ArtistName et Categorie
  24/12/2002 V1.03: Modif du code de SetHttpConnection et Read(...) pour
                    compatibilit avec Indy 9
}

const
  Ver = '1.03 - 24/12/2002';

type
  TDataEvent = procedure(Sender : TObject; AData : string) of object;
  TDataReceiveEvent = procedure(Sender : TObject; Code: Integer; AData : string) of object;

  TCDDBConnect = class(TComponent)
    private
      FVersion: String;
      FDrive: Char;
      FDriveList: TStrings;
      FCDDBDiscIdHex: String;
      FCDTotalTrackCount: Integer;
      FCDTotalLenght: Integer;
      FServer: string;
      FClientVersion: string;
      FClientName: string;
      FHostName: string;
      FUserName: string;
      FRetourServeurCDDB: String;
      FHTTPConnection : TIdHTTP;
      FLastError : integer;
      FPort: Integer;
      FProxyServer: String;
      FProxyPort: Integer;
      FProxyPassword: String;
      FProxyUsername: String;
      FOnSendData: TDataEvent;
      FOnReceiveData: TDataReceiveEvent;
      DummyString: String;
      ReponseServeur: TStringList;
      function GenerateCmd(CmdName: string): string;
      function GetDriveList: TStrings;
      function GetAlbumName: String;
      function GetArtistName: String;
      function GetCategorie: String;
    protected
      procedure SetHttpConnection; virtual;
    public
      constructor Create (AOwner : TComponent); override;
      destructor Destroy; override;
      function SendCommand(ACmd : string) : string;
      function Help(ACmd : string) : string;
      function Sites : string;
      function MessageOfTheDay : string;
      function Stats : string;
      function InfoForCurrentCD(Drive: Char; var CodeErreur: Integer): string;
      function Read(CdInfo: String): String;
      function Users : string;
      procedure RefreshCD;
      // S'il y a un seul album, retourne la liste des pistes
      procedure GetTrackNameList(Liste: TStrings);
      // Et des extensions ventuelles
      procedure GetTrackExtList(Liste: TStrings);
      // Album
      property AlbumName: String read GetAlbumName;
      // Artiste
      property ArtistName: String read GetArtistName;
      // Catgorie
      property Categorie: String read GetCategorie;
      // Code de la derniere "erreur"
      property LastError : integer read FLastError;
      // Proprit du disc
      property CDDBDiscIdHex: String read FCDDBDiscIdHex;
      property CDTotalTrackCount: Integer read FCDTotalTrackCount;
      property CDTotalLenght: Integer read FCDTotalLenght;
      // Liste des lecteurs CD du systme
      property DriveList: TStrings read GetDriveList;
    published
      property ComponentVersion: String read FVersion write DummyString;
      // Lettre du lecteur
      property Drive: Char write FDrive;
      // Proprit du serveur CDDB
      property UserName : string read FUserName write FUserName;
      property HostName : string read FHostName write FHostName;
      property ClientName : string read FClientName write FClientName;
      property ClientVersion : string read FClientVersion write FClientVersion;
      property Server : string read FServer write FServer;
      // Proprits de connexion HTTP
      property Port: Integer read FPort write FPort default 8880;
      property ProxyServer: String read FProxyServer write FProxyServer;
      property ProxyPort: Integer read FProxyPort write FProxyPort default 80;
      property ProxyPassword: String read FProxyPassword write FProxyPassword;
      property ProxyUsername: String read FProxyUsername write FProxyUsername;
      // Evenements
      property OnSendData : TDataEvent read FOnSendData write FOnSendData;
      property OnReceiveData : TDataReceiveEvent read FOnReceiveData write FOnReceiveData;
  end;

procedure Register;


implementation
uses
  Windows, Forms, Controls, CDInfo, SysUtils, Math;

const
  CPlusSep = '+';

procedure Register;
begin
  RegisterComponents('FD M.c.s', [TCDDBConnect]);
end;

function RemoveToken(const AStr :string; Token : char) : string;
var
  Loop : integer;
begin
  Result := '';
  for Loop := 1 to Length(AStr) do
    if AStr[Loop] <> Token then
      Result := Result + Astr[Loop];
end;

function FileNoExtension(const AFilename: String): String;
var
  JustFile: String;
  JustFileWithoutExt: String;
begin
  JustFile := ExtractFilename(AFilename);
  if Pos('.', JustFile) = 0 then JustFileWithoutExt := JustFile
                            else JustFileWithoutExt := Copy(JustFile, 1, Length(JustFile) - 4);
  Result := JustFileWithoutExt;
end;

{ TCDDBConnect }
constructor TCDDBConnect.Create(AOwner : TComponent);
begin
  inherited;
  Fversion := Ver;
  FDriveList := TStringList.Create;
  ReponseServeur := TStringList.Create;

  FCDDBDiscIdHex := '';
  FCDTotalTrackCount := 0;

  FPort := 8880;
  FProxyServer := '';
  FProxyPort := 80;
  FProxyPassword := '';
  FProxyUsername := '';

  FHTTPConnection := TIdHTTP.Create(nil);
  FUserName := 'Unknown';
  FClientVersion := '1.0';
  FClientName := 'Unknown';
  FHostName := 'Unknown';
  FServer := 'http://us.cddb.com/~cddb';
end;

destructor TCDDBConnect.Destroy;
begin
  ReponseServeur.Free;
  FHTTPConnection.Free;
  inherited;
end;

procedure TCDDBConnect.SetHttpConnection;
begin
  With FHTTPConnection do begin
    Port := FPort;
    // Modifi le 24-12-2002 pour compatibilit avec Indy 9
    ProxyParams.ProxyServer := FProxyServer;
    ProxyParams.ProxyPort := FProxyPort;
    ProxyParams.ProxyUsername := FProxyUsername;
    ProxyParams.ProxyPassword := FProxyPassword;

    { Ancien code pour Indy 8
    Request.ProxyServer := FProxyServer;
    Request.ProxyPort := FProxyPort;
    Request.ProxyUsername := FProxyUsername;
    Request.ProxyPassword := FProxyPassword;
    }

  end;
end;

// Liste des lecteur CDRom du systme (Lettre)
function TCDDBConnect.GetDriveList: TStrings;
var
  n: Integer;
begin
  FDriveList.Clear;
  for n := 1 to 26 do
    if (GetDriveType(PChar(Char(n + 64) + ':\')) = DRIVE_CDROM) then FDriveList.Add(Char(n + 64));
  Result := FDriveList;
end;

// Envoie rellement la commande vers le serveur CDDB
function TCDDBConnect.SendCommand(ACmd: string): string;
var
  LFullCmd : string;
  LMemStrm : TMemoryStream;
  OldCursor: TCursor;
begin
  OldCursor := Screen.Cursor;
  Screen.Cursor := crHourglass;
  LMemStrm := TMemoryStream.Create;
  try
    LFullCmd := GenerateCmd(ACmd);
    SetHttpConnection;
    if Assigned(FOnSendData) then FOnSendData(Self, LFullCmd);
    FHTTPConnection.Get(LFullCmd, LMemStrm);
    LMemStrm.Position := 0;
    Result := Copy(PChar(LMemStrm.Memory), 1, LMemStrm.Size);
    FLastError := StrToInt(Copy(Result, 1, 3));
  finally
    FHTTPConnection.Disconnect;
    if Assigned(FOnReceiveData) then FOnReceiveData(Self, FLastError, Result);
    LMemStrm.Free;
    Screen.Cursor := OldCursor;
  end;
end;

// Gnre le chaine de connection correcte
function TCDDBConnect.GenerateCmd(CmdName : string) : string;
begin
  Result := FServer + '/cddb.cgi?cmd=' + CmdName +
    '&hello=' + FUserName + CPlusSep + FHostName + CPlusSep + FClientName + CPlusSep
    + FClientVersion + '&proto=4';
end;

// Commande Help
function TCDDBConnect.Help(ACmd: string): string;
begin
  Result := SendCommand('help' + CPlusSep + ACmd);
end;

// Commande 'Sites' Retourne des info sur le serveur
function TCDDBConnect.Sites: string;
begin
  Result := SendCommand('sites');
end;

// Message du jour (dernires MAJ du serveur)
function TCDDBConnect.MessageOfTheDay: string;
begin
  Result := SendCommand('motd');
end;

// Stat du serveur
function TCDDBConnect.Stats: string;
begin
  Result := SendCommand('stat');
end;

function TCDDBConnect.Users: string;
begin
  Result := SendCommand('whom');
end;

// Info sur le CD courant
function TCDDBConnect.InfoForCurrentCD(Drive: Char; var CodeErreur: Integer): string;
var
  LCDInfo : TCDInfo;
  Commande : string;
  Loop : integer;
begin
  LCDInfo := TCDInfo.Create;
  try
    LCDInfo.OpenDevice(Drive);
    FCDDBDiscIdHex := LCDInfo.CDDBDiscIdHex;
    FCDTotalTrackCount := LCDInfo.TotalTrackCount;
    FCDTotalLenght := LCDInfo.TotalCDLength;
    // Interroge la CDDB en premiers
    Commande := 'cddb' + CPlusSep + 'query' + CPlusSep + FCDDBDiscIdHex
          + '+' + IntToStr(FCDTotalTrackCount);
    for Loop := 1 to FCDTotalTrackCount do
      Commande := Commande + CPlusSep + IntToStr(LCDInfo.TOCToFrames(LCDInfo.TrackPosition(Loop)));
    Commande := Commande + CPlusSep + IntToStr(FCDTotalLenght);
    Result := Trim(SendCommand(Commande));
    { Si une seul rponse on enlve le code Ok (200)
      il Faudra ensuite utiliser la commande Read avec le resultat}
    CodeErreur := LastError;
    if LastError = 200 then result := Read(Copy(Result, Pos(' ', Result)+1, Length(Result)));
  finally
    LCDInfo.Free;
  end;
end;

function TCDDBConnect.Read(CdInfo: String): String;
var
  Categ: String;
  DiscId: String;
begin
  //Ajouter le 24-12-2002 pour compatibilit avec Indy 9.
  if (Copy(CdInfo, 1 ,3) = '210') or (Copy(CdInfo, 1 ,3) = '200') then CdInfo := Trim(Copy(CdInfo, 4, 999));
  Categ := Copy(CdInfo, 1, Pos(' ', CdInfo) -1);
  DiscID := Copy(CdInfo, Pos(' ', CdInfo) +1, Length(CdInfo));
  DiscId := Copy(DiscId, 1, Pos(' ', DiscId) -1);
  FRetourServeurCDDB := SendCommand('cddb' + CPlusSep + 'read' + CPlusSep + Categ + CPlusSep + DiscId + CPlusSep + '');
  ReponseServeur.Text := FRetourServeurCDDB;
  Result := FRetourServeurCDDB;
end;

procedure TCDDBConnect.GetTrackNameList(Liste: TStrings);
var
  ReponseServeur: TStringList;
  n: Integer;
begin
  Liste.Clear;
  ReponseServeur := TStringList.Create;
  With ReponseServeur do try
    ReponseServeur.Text := FRetourServeurCDDB;
    for n := 0 to Count -1 do
      if UpperCase(Copy(Strings[n], 1, 6)) = 'TTITLE' then
        Liste.Add(Copy(Strings[n], Pos('=', Strings[n]) +1, Length(Strings[n])));
  finally
    Free;
  end;
end;

procedure TCDDBConnect.GetTrackExtList(Liste: TStrings);
var
  ReponseServeur: TStringList;
  n: Integer;
begin
  Liste.Clear;
  ReponseServeur := TStringList.Create;
  With ReponseServeur do try
    ReponseServeur.Text := FRetourServeurCDDB;
    for n := 0 to Count -1 do
      if UpperCase(Copy(Strings[n], 1, 4)) = 'EXTT' then
        Liste.Add(Copy(Strings[n], Pos('=', Strings[n]) +1, Length(Strings[n])));
  finally
    Free;
  end;
end;

function TCDDBConnect.GetAlbumName: String;
var
  n: Integer;
begin
  Result := '';
  With ReponseServeur do begin
    for n := 0 to Count -1 do
      if UpperCase(Copy(Strings[n], 1, 6)) = 'DTITLE' then
        Result := Trim(Copy(Strings[n], Pos('/', Strings[n]) +1, Length(Strings[n])));
  end;
end;

function TCDDBConnect.GetArtistName: String;
var
  n: Integer;
begin
  Result := '';
  With ReponseServeur do begin
    for n := 0 to Count -1 do
      if UpperCase(Copy(Strings[n], 1, 6)) = 'DTITLE' then
        Result := Trim(Copy(Strings[n], Pos('=', Strings[n]) +1, Pos('/', Strings[n]) - Pos('=', Strings[n]) -1));
  end;
end;

function TCDDBConnect.GetCategorie: String;
begin
  With ReponseServeur do
    Result := Copy(Strings[0], Pos(' ', Strings[0]) +1, Length(Strings[0]));
  Result := Copy(Result, 1, Pos(' ', Result) -1);
end;

procedure TCDDBConnect.RefreshCD;
var
  LCDInfo : TCDInfo;
begin
  LCDInfo := TCDInfo.Create;
  try
    LCDInfo.OpenDevice(FDrive);
    FCDDBDiscIdHex := LCDInfo.CDDBDiscIdHex;
    FCDTotalTrackCount := LCDInfo.TotalTrackCount;
    FCDTotalLenght := LCDInfo.TotalCDLength;
  finally
    LCDInfo.Free;
  end;
end;

end.
